//CCVisu is a tool for co-change visualization.
//This file is part of CCVisu. 

//Copyright (C) 2005-2006  Dirk Beyer

//CCVisu is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public License
//as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.

//CCVisu is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public License
//along with CCVisu; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//Please find the GNU Lesser General Public License in file
//../license_lgpl.txt or http://www.gnu.org/licenses/lgpl.txt

//Dirk Beyer    (firstname.lastname@epfl.ch)
//Ecole Polytechnique Federale de Lausanne (EPFL), Switzerland


package ccvisu;

/*****************************************************************
 * Marks a given vertex under certain conditions.
 * @version  $Revision: 1.4 $; $Date: 2006/05/29 09:55:15 $
 * @author   Dirk Beyer
 *****************************************************************/
public class MarkerExp extends Marker{
  /** Constructor.*/
  public MarkerExp() {
  }

  /*****************************************************************
   * Special marking for certain vertices by setting attributes of the vertex,
   * e.g., color and showName. 
   * This method should be overwritten by a user-defined subclass in order to 
   * implement some 'highlighting' functionality.
   * @param curVertex  Vertex of the graph representation.
   *****************************************************************/
  public void mark(GraphVertex curVertex) {
	    
	    // Colors for Mozilla.
	    if (curVertex.name.startsWith("imap/")) {
	        curVertex.color = CCVisu.pink;
	    } else if (curVertex.name.startsWith("db/")) {
	        curVertex.color = CCVisu.orange;
	    } else if (curVertex.name.startsWith("mapi/")) {
	        curVertex.color = CCVisu.yellow;
	    } else if (curVertex.name.startsWith("compose/")) {
	        curVertex.color = CCVisu.magenta;
	    } else if (curVertex.name.startsWith("addrbook/")) {
	        curVertex.color = CCVisu.blue;
	    } else if (curVertex.name.startsWith("import/")) {
	        curVertex.color = CCVisu.cyan;
	    } else if (curVertex.name.startsWith("mime/")) {
	        curVertex.color = CCVisu.red;
	    } else if (curVertex.name.startsWith("extensions/")) {
	        curVertex.color = CCVisu.green;
	    } else if (curVertex.name.startsWith("news/")) {
	        curVertex.color = CCVisu.chocolate4;
	    } else {
	        curVertex.color = CCVisu.lightGray;
	    }
        
	  
	    /*
	    // Colors for ArgoUML.
	    if (curVertex.name.startsWith("src/uci/gef/")) {
	        curVertex.color = chocolate4;
	    //} else if (curVertex.name.startsWith("src/uci/uml/ui/")) {
	    //    curVertex.color = CCVisu.DarkOliveGreen4;
	    } else if (curVertex.name.startsWith("src/uci/")) {
	        curVertex.color = green;
	    } else if (curVertex.name.matches(".*documentation/.*")) {
	        curVertex.color = yellow;
	    } else if (curVertex.name.startsWith("tests/")) {
	        curVertex.color = blue;
	    } else if (curVertex.name.matches(".*cognitive/.*")) {
	        curVertex.color = cyan;
	    } else if (curVertex.name.matches(".*diagram/.*")) {
	        curVertex.color = magenta;
	    } else if (curVertex.name.startsWith("src_new/org/argouml/ui/")) {
	        curVertex.color = darkCyan;
	    } else if (curVertex.name.startsWith("src_new/org/argouml/uml/ui/")) {
	        curVertex.color = red;
	    } else if (curVertex.name.startsWith("src_new/org/argouml/model/")) {
	        curVertex.color = lightBlue;
	    } else {
	        curVertex.color = lightGray;
	    }
	    */
	                 
	    /*
	    // Colors for crocopat.
	    if (curVertex.name.startsWith("programs/")) {
	          curVertex.color = Color.GREEN;
	    } else if (curVertex.name.startsWith("src/bdd")) {
	          curVertex.color = Color.CYAN;
	    } else if (curVertex.name.startsWith("src/cta")) {
	          curVertex.color = Color.CYAN;
	    } else if (curVertex.name.startsWith("src/repr")) {
	          curVertex.color = Color.CYAN;
	    } else if (curVertex.name.startsWith("RelExtractor/")) {
	          curVertex.color = Color.GRAY;
	    } else if (curVertex.name.startsWith("projects/")) {
	          curVertex.color = Color.RED;
	    } else if (curVertex.name.startsWith("src/rel")) {
	          curVertex.color = Color.MAGENTA;
	    } else if (curVertex.name.startsWith("src/Felx")) {
	          curVertex.color = Color.MAGENTA;
	    } else {
	          curVertex.color = Color.BLUE;
	    }
	    */
	        
	    /*      
	    // Colors for rabbit.
	    if (curVertex.name.startsWith("ana/")) {
	      curVertex.color = Color.GREEN;
	    } else if (curVertex.name.startsWith("bdd/")) {
	      curVertex.color = Color.BLUE;
	    } else if (curVertex.name.startsWith("example/")) {
	      curVertex.color = Color.GRAY;
	    } else if (curVertex.name.startsWith("ddm/")) {
	      curVertex.color = Color.MAGENTA;
	    } else if (curVertex.name.startsWith("cta/")) {
	      curVertex.color = Color.RED;
	    } else if (curVertex.name.startsWith("doc/")) {
	      curVertex.color = Color.CYAN;
	    } else {
	      curVertex.color = Color.YELLOW;
	    }
	    */

	    /*
	    // Colors for postgreSQL.
	    if (curVertex.name.startsWith("doc/")) {
	        curVertex.color = green;
	    } else if (curVertex.name.startsWith("src/backend/executor/")) {
	        curVertex.color = red;
	    } else if (curVertex.name.startsWith("src/backend/optimizer/")) {
	        curVertex.color = blue;
	    } else if (curVertex.name.startsWith("src/backend/parser/")) {
	        curVertex.color = cyan;
	    } else if (curVertex.name.startsWith("src/backend/storage/")) {
	        curVertex.color = magenta;
	    } else if (curVertex.name.startsWith("src/backend/catalog/")) {
	        curVertex.color = yellow;
	    } else if (curVertex.name.startsWith("src/backend/commands/")) {
	        curVertex.color = yellow;
	    } else if (curVertex.name.startsWith("src/backend/nodes/")) {
	        curVertex.color = yellow;
	    } else if (curVertex.name.startsWith("src/backend/access/")) {
	        curVertex.color = darkCyan;
	    } else if (curVertex.name.startsWith("src/backend/port/dynloader/")) {
	        curVertex.color = darkOliveGreen4;
	    } else if (curVertex.name.startsWith("src/test/")) {
	        curVertex.color = chocolate4;
	    } else if (curVertex.name.startsWith("src/interfaces/")) {
	        curVertex.color = lightBlue;
	    } else if (curVertex.name.startsWith("src/include/")) {
	        curVertex.color = lightGreen;
//	    } else if (curVertex.name.startsWith("src/backend/utils/")) {
//	        curVertex.color = lightGray;
//	    } else if (curVertex.name.startsWith("src/backend/")) {
//	        curVertex.color = pink;
	    } else {
	        curVertex.color = lightGray;
	    }
	    */

	    /*
	    // Colors for postgreSQL-backend-lifted.
	    if (curVertex.name.startsWith("src/backend/port/")) {
	      curVertex.color = Color.RED;
	    } else if (curVertex.name.startsWith("src/backend/nodes/")) {
	        curVertex.color = Color.PINK;
	    } else if (curVertex.name.startsWith("src/backend/utils/adt/")) {
	      curVertex.color = Color.CYAN;
	    } else if (curVertex.name.startsWith("src/backend/storage/")) {
	        curVertex.color = Color.BLUE;
	    } else if (curVertex.name.startsWith("src/backend/optimizer/")) {
	        curVertex.color = Color.GREEN;
	    } else if (curVertex.name.startsWith("src/backend/parser/")) {
	        curVertex.color = Color.YELLOW;
	    } else if (curVertex.name.startsWith("src/backend/access/")) {
	      curVertex.color = Color.MAGENTA;
	    } else if (curVertex.name.startsWith("src/backend/executor/")) {
	      curVertex.color = Color.GRAY;
	    } else {
	      curVertex.color = Color.LIGHT_GRAY;
	    }
	    */

  }

};


